@@ -1,11 +1,8 @@
-use archive::Struct;
-use handle::Handle;
-use storage::MemoryDescriptor;
+use crate::archive::Struct;
 
 use std::fmt;
 use std::iter;
 use std::marker;
-use std::slice;
 
 /// A read-only view on a contiguous sequence of flatdata structs of the same
 /// type `T`.
@@ -24,7 +21,12 @@ use std::slice;
 /// # fn main() {
 /// use flatdata::{ArrayView, Vector};
 ///
-/// define_struct!(A, AMut, "no_schema", 4,
+/// define_struct!(
+///     A,
+///     RefA,
+///     RefMutA,
+///     "no_schema",
+///     4,
 ///     (x, set_x, u32, 0, 16),
 ///     (y, set_y, u32, 16, 16)
 /// );
@@ -46,25 +48,31 @@ use std::slice;
 /// [`Vector`]: struct.Vector.html
 /// [`ExternalVector`]: struct.ExternalVector.html
 #[derive(Clone)]
-pub struct ArrayView<'a, T: 'a> {
+pub struct ArrayView<'a, T>
+where
+    T: for<'b> Struct<'b>,
+{
     data: &'a [u8],
     _phantom: marker::PhantomData<T>,
 }
 
-impl<'a, T: Struct> ArrayView<'a, T> {
+impl<'a, T> ArrayView<'a, T>
+where
+    T: for<'b> Struct<'b>,
+{
     /// Creates a new `ArrayView` to the data at the given address.
     ///
     /// The returned array view does not own the data.
-    pub fn new(mem_descr: &MemoryDescriptor) -> Self {
+    pub fn new(data: &'a [u8]) -> Self {
         Self {
-            data: unsafe { slice::from_raw_parts(mem_descr.data(), mem_descr.size_in_bytes()) },
+            data,
             _phantom: marker::PhantomData,
         }
     }
 
     /// Number of elements in the array.
     pub fn len(&self) -> usize {
-        self.data.len() / T::SIZE_IN_BYTES
+        self.data.len() / <T as Struct>::SIZE_IN_BYTES
     }
 
     /// Return `true` if the array is empty.
@@ -78,16 +86,16 @@ impl<'a, T: Struct> ArrayView<'a, T> {
     /// # Panics
     ///
     /// Panics if index is greater than or equal to `ArrayView::len()`.
-    pub fn at(&self, index: usize) -> Handle<'a, T> {
-        let index = index * T::SIZE_IN_BYTES;
-        assert!(index + T::SIZE_IN_BYTES <= self.data.len());
-        Handle::new(T::from(&self.data[index]))
+    pub fn at(&self, index: usize) -> <T as Struct<'a>>::Item {
+        let index = index * <T as Struct>::SIZE_IN_BYTES;
+        assert!(index + <T as Struct>::SIZE_IN_BYTES <= self.data.len());
+        T::create(&self.data[index..])
     }
 
     /// Returns an iterator to the elements of the array.
-    pub fn iter(&'a self) -> ArrayViewIter<T> {
+    pub fn iter(&self) -> ArrayViewIter<'a, T> {
         ArrayViewIter {
-            view: self,
+            view: self.clone(),
             next_pos: 0,
         }
     }
@@ -98,7 +106,10 @@ impl<'a, T: Struct> ArrayView<'a, T> {
     }
 }
 
-impl<'a, T: Struct> fmt::Debug for ArrayView<'a, T> {
+impl<'a, T> fmt::Debug for ArrayView<'a, T>
+where
+    T: for<'b> Struct<'b>,
+{
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         let preview: Vec<_> = self.iter().take(super::DEBUG_PREVIEW_LEN).collect();
         write!(
@@ -115,7 +126,10 @@ impl<'a, T: Struct> fmt::Debug for ArrayView<'a, T> {
     }
 }
 
-impl<'a, T: Struct> AsRef<[u8]> for ArrayView<'a, T> {
+impl<'a, T> AsRef<[u8]> for ArrayView<'a, T>
+where
+    T: for<'b> Struct<'b>,
+{
     fn as_ref(&self) -> &[u8] {
         self.as_bytes()
     }
@@ -123,13 +137,19 @@ impl<'a, T: Struct> AsRef<[u8]> for ArrayView<'a, T> {
 
 /// Iterator through elements of `ArrayView`.
 #[derive(Clone)]
-pub struct ArrayViewIter<'a, T: 'a + Struct> {
-    view: &'a ArrayView<'a, T>,
+pub struct ArrayViewIter<'a, T>
+where
+    T: for<'b> Struct<'b>,
+{
+    view: ArrayView<'a, T>,
     next_pos: usize,
 }
 
-impl<'a, T: Struct> iter::Iterator for ArrayViewIter<'a, T> {
-    type Item = Handle<'a, T>;
+impl<'a, T> iter::Iterator for ArrayViewIter<'a, T>
+where
+    T: for<'b> Struct<'b>,
+{
+    type Item = <T as Struct<'a>>::Item;
     fn next(&mut self) -> Option<Self::Item> {
         if self.next_pos < self.view.len() {
             let element = self.view.at(self.next_pos);
@@ -141,15 +161,22 @@ impl<'a, T: Struct> iter::Iterator for ArrayViewIter<'a, T> {
     }
 }
 
-impl<'a, T: Struct> iter::ExactSizeIterator for ArrayViewIter<'a, T> {
+impl<'a, T> iter::ExactSizeIterator for ArrayViewIter<'a, T>
+where
+    T: for<'b> Struct<'b>,
+{
     fn len(&self) -> usize {
         self.view.len()
     }
 }
 
-impl<'a, T: Struct> fmt::Debug for ArrayViewIter<'a, T> {
+impl<'a, T> fmt::Debug for ArrayViewIter<'a, T>
+where
+    T: for<'b> Struct<'b>,
+{
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        let preview = self.view
+        let preview = self
+            .view
             .iter()
             .skip(self.next_pos)
             .take(super::DEBUG_PREVIEW_LEN);
@@ -165,3 +192,52 @@ impl<'a, T: Struct> fmt::Debug for ArrayViewIter<'a, T> {
         )
     }
 }
+
+#[cfg(test)]
+mod test {
+    use crate::archive::Struct;
+    use crate::memory;
+
+    #[test]
+    #[allow(dead_code)]
+    fn test() {
+        define_struct!(
+            A,
+            RefA,
+            RefMutA,
+            "no_schema",
+            4,
+            (x, set_x, u32, 0, 16),
+            (y, set_y, u32, 16, 16)
+        );
+
+        let mut buffer = vec![255_u8; 4];
+        buffer.extend(vec![0_u8; A::SIZE_IN_BYTES * 10 + memory::PADDING_SIZE]);
+        let data = &buffer[..buffer.len() - memory::PADDING_SIZE];
+        let view: super::ArrayView<A> = super::ArrayView::new(&data);
+        assert_eq!(11, view.len());
+        let first = view.at(0);
+        assert_eq!(65535, first.x());
+        assert_eq!(65535, first.y());
+        for x in view.iter().skip(1) {
+            assert_eq!(0, x.x());
+            assert_eq!(0, x.y());
+        }
+
+        let x = {
+            // test clone and lifetime of returned reference
+            let view_copy = view.clone();
+            view_copy.at(0)
+        };
+        assert_eq!(65535, x.x());
+        assert_eq!(65535, x.y());
+
+        let x = {
+            // test clone and lifetime of returned reference
+            let view_copy = view.clone();
+            view_copy.iter().next().unwrap()
+        };
+        assert_eq!(65535, x.x());
+        assert_eq!(65535, x.y());
+    }
+}
